/**
 * CeboMsr - API, header.
 *
 * (c) 2012 Cesys GmbH
 * @author Thomas Hoppe
 */
#ifndef	__CEBOMSR_INTERFACE_H__
#define	__CEBOMSR_INTERFACE_H__

// DLL import/export macro.
#ifdef	WIN32
#	ifdef	CEBOMSR_STATIC
#		define	DLLEXPORT
#	else
#		ifdef	CEBOMSR_EXPORTS
#			define	DLLEXPORT	__declspec(dllexport) _cdecl
#		else
#			define	DLLEXPORT	__declspec(dllimport) _cdecl
#		endif
#	endif
#else
#	define	DLLEXPORT
#endif

#ifdef _MSC_VER
	typedef __int32 int32_t;
	typedef unsigned __int32 uint32_t;
	typedef __int64 int64_t;
	typedef unsigned __int64 uint64_t;
#else
#	include <stdint.h>
#endif

// Use flat C interface.
#ifdef	__cplusplus
extern "C" {
#endif

typedef int CeboMsrType;
const CeboMsrType CeboMsrTypeAll = 0xceb00000;
const CeboMsrType CeboMsrTypeUsb = 0xceb00001;
const CeboMsrType CeboMsrTypeCeboLC = 0xceb00010;
const CeboMsrType CeboMsrTypeCeboStick = 0xceb00011;

const int CeboMsrMaxIdentLength = 1024;

const int CeboParameterSingleEndedCount = 0;
const int CeboParameterDifferentialCount = 1;
const int CeboParameterAnalogOutputCount = 2;
const int CeboParameterDigitalPortCount = 3;
const int CeboParameterCounterCount = 4;
const int CeboParameterTriggerCount = 5;
const int CeboParameterMaxInputFrames = 6;
const int CeboParameterCurrentSourceCount = 7;
const int CeboParameterLedCount = 8;

typedef int *DigitalArray;
typedef float *VoltageArray;

typedef int CeboMsrErrorIndex;

typedef int CeboMsrHandle;

typedef int CeboMsrIo;
const CeboMsrIo CeboMsrIoIdDigitalPort0 = 0xceb01000;
const CeboMsrIo CeboMsrIoIdDigitalPort1 = 0xceb01001;
const CeboMsrIo CeboMsrIoIdDigitalPort2 = 0xceb01002;
const CeboMsrIo CeboMsrIoIdDigitalPort3 = 0xceb01003;
const CeboMsrIo CeboMsrIoIdDigitalPort4 = 0xceb01004;
const CeboMsrIo CeboMsrIoIdDigitalPort5 = 0xceb01005;
const CeboMsrIo CeboMsrIoIdDigitalPort6 = 0xceb01006;
const CeboMsrIo CeboMsrIoIdDigitalPort7 = 0xceb01007;

const CeboMsrIo CeboMsrIoIdSingleEnded0 = 0xceb01100;
const CeboMsrIo CeboMsrIoIdSingleEnded1 = 0xceb01101;
const CeboMsrIo CeboMsrIoIdSingleEnded2 = 0xceb01102;
const CeboMsrIo CeboMsrIoIdSingleEnded3 = 0xceb01103;
const CeboMsrIo CeboMsrIoIdSingleEnded4 = 0xceb01104;
const CeboMsrIo CeboMsrIoIdSingleEnded5 = 0xceb01105;
const CeboMsrIo CeboMsrIoIdSingleEnded6 = 0xceb01106;
const CeboMsrIo CeboMsrIoIdSingleEnded7 = 0xceb01107;
const CeboMsrIo CeboMsrIoIdSingleEnded8 = 0xceb01108;
const CeboMsrIo CeboMsrIoIdSingleEnded9 = 0xceb01109;
const CeboMsrIo CeboMsrIoIdSingleEnded10 = 0xceb0110a;
const CeboMsrIo CeboMsrIoIdSingleEnded11 = 0xceb0110b;
const CeboMsrIo CeboMsrIoIdSingleEnded12 = 0xceb0110c;
const CeboMsrIo CeboMsrIoIdSingleEnded13 = 0xceb0110d;
const CeboMsrIo CeboMsrIoIdSingleEnded14 = 0xceb0110e;
const CeboMsrIo CeboMsrIoIdSingleEnded15 = 0xceb0110f;

const CeboMsrIo CeboMsrIoIdDifferential0 = 0xceb01200;
const CeboMsrIo CeboMsrIoIdDifferential1 = 0xceb01201;
const CeboMsrIo CeboMsrIoIdDifferential2 = 0xceb01202;
const CeboMsrIo CeboMsrIoIdDifferential3 = 0xceb01203;
const CeboMsrIo CeboMsrIoIdDifferential4 = 0xceb01204;
const CeboMsrIo CeboMsrIoIdDifferential5 = 0xceb01205;
const CeboMsrIo CeboMsrIoIdDifferential6 = 0xceb01206;
const CeboMsrIo CeboMsrIoIdDifferential7 = 0xceb01207;

const CeboMsrIo CeboMsrIoIdAnalogOutput0 = 0xceb01300;
const CeboMsrIo CeboMsrIoIdAnalogOutput1 = 0xceb01301;

const CeboMsrIo CeboMsrIoIdCounter0 = 0xceb01500;

const CeboMsrIo CeboMsrIoIdTrigger0 = 0xceb01600;

typedef int CeboMsrRange;
const CeboMsrRange CeboMsrRange0 = 0xceb01400;
const CeboMsrRange CeboMsrRange1 = 0xceb01401;
const CeboMsrRange CeboMsrRange2 = 0xceb01402;
const CeboMsrRange CeboMsrRange3 = 0xceb01403;
const CeboMsrRange CeboMsrRangeInvalid = 0xceb014ff;

typedef int CeboMsrCurrentSource;
const CeboMsrCurrentSource CeboMsrCurrentSource0 = 0xceb01700;
const CeboMsrCurrentSource CeboMsrCurrentSource1 = 0xceb01701;

typedef int CeboMsrLed;
const CeboMsrLed CeboMsrLedIndicator = 0xceb01800;

const int CeboMsrPeripheralResetMaskInputFifo = 0x00000001;

typedef int CeboMsrTriggerConfigId;
const CeboMsrTriggerConfigId CeboMsrTriggerOutputPulse = 0xceb01900;
const CeboMsrTriggerConfigId CeboMsrTriggerOutputAlternating = 0xceb01901;
const CeboMsrTriggerConfigId CeboMsrTriggerInputRisingEdge = 0xceb01902;
const CeboMsrTriggerConfigId CeboMsrTriggerInputFallingEdge = 0xceb01903;
const CeboMsrTriggerConfigId CeboMsrTriggerInputAlternating = 0xceb01904;

typedef int CeboMsrCounterConfigId;
const CeboMsrCounterConfigId CeboMsrCounterConfigRisingEdge = 0xceb01a00;
const CeboMsrCounterConfigId CeboMsrCounterConfigFallingEdge = 0xceb01a01;
const CeboMsrCounterConfigId CeboMsrCounterConfigAlternating = 0xceb01a02;

/// Error handling.
int DLLEXPORT CeboMsrGetErrorLength(CeboMsrErrorIndex index);
void DLLEXPORT CeboMsrGetError(CeboMsrErrorIndex index, void *buffer);

/// Host version information.
CeboMsrErrorIndex DLLEXPORT CeboMsrGetUsbBaseVersion(char *buffer);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetApiVersion(char *buffer);

/// Enumeration.
CeboMsrErrorIndex DLLEXPORT CeboMsrBeginEnumerate(int *count, CeboMsrType type);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetIdentifier(int id, char *ident, CeboMsrType *type, char *version);

/// Access.
CeboMsrErrorIndex DLLEXPORT CeboMsrOpen(char *ident, CeboMsrHandle *handle);
CeboMsrErrorIndex DLLEXPORT CeboMsrClose(CeboMsrHandle handle);
CeboMsrErrorIndex DLLEXPORT CeboMsrResetDevice(CeboMsrHandle handle);
CeboMsrErrorIndex DLLEXPORT CeboMsrResetPeripherals(CeboMsrHandle handle, int mask);

/// Box specific.
CeboMsrErrorIndex DLLEXPORT CeboMsrGetDeviceParameter(CeboMsrHandle handle, int parameter, int *value);
CeboMsrErrorIndex DLLEXPORT CeboMsrEnumerateRanges(CeboMsrHandle handle, CeboMsrIo ioId, int id, CeboMsrRange *rangeId);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetRangeMinMax(CeboMsrHandle handle, CeboMsrIo ioId, CeboMsrRange rangeId, float *minValue, float *maxValue);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetDigitalPortIoCount(CeboMsrHandle handle, CeboMsrIo ioId, int *ioCount);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetPeripheralName(CeboMsrHandle handle, int id, char *name);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetDigitalPortIoName(CeboMsrHandle handle, CeboMsrIo ioId, int io, char *name);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetDefaultAnalogInputInterChannelDelay(CeboMsrHandle handle, CeboMsrIo ioId, CeboMsrRange rangeId, int *defaultInterChannelDelay);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetMinAnalogInputInterChannelDelay(CeboMsrHandle handle, CeboMsrIo ioId, int *minInterChannelDelay);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetSerialNumber(CeboMsrHandle handle, char *buffer);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetTemperature(CeboMsrHandle handle, float *temperature);
CeboMsrErrorIndex DLLEXPORT CeboMsrSetLed(CeboMsrHandle handle, CeboMsrLed led, int state);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetReferenceCurrent(CeboMsrHandle handle, CeboMsrCurrentSource source, float *current);

/// Configuration.
CeboMsrErrorIndex DLLEXPORT CeboMsrSetDigitalPortOutputEnableMask(CeboMsrHandle handle, CeboMsrIo ioId, int mask);
CeboMsrErrorIndex DLLEXPORT CeboMsrSetAnalogInputParameters(CeboMsrHandle handle, CeboMsrIo ioId, CeboMsrRange rangeId, int interChannelDelay, int *correctedInterChannelDelay);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetAnalogInputParameters(CeboMsrHandle handle, CeboMsrIo ioId, CeboMsrRange *rangeId, int *interChannelDelay);
CeboMsrErrorIndex DLLEXPORT CeboMsrSetTriggerConfig(CeboMsrHandle handle, CeboMsrIo ioId, CeboMsrTriggerConfigId triggerConfigId);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetTriggerConfig(CeboMsrHandle handle, CeboMsrIo ioId, CeboMsrTriggerConfigId *triggerConfigId);
CeboMsrErrorIndex DLLEXPORT CeboMsrSetTriggerEnable(CeboMsrHandle handle, CeboMsrIo ioId, int enable);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetTriggerEnable(CeboMsrHandle handle, CeboMsrIo ioId, int *enable);
CeboMsrErrorIndex DLLEXPORT CeboMsrResetCounter(CeboMsrHandle handle, CeboMsrIo ioId);
CeboMsrErrorIndex DLLEXPORT CeboMsrSetCounterConfig(CeboMsrHandle handle, CeboMsrIo ioId, CeboMsrCounterConfigId counterConfigId);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetCounterConfig(CeboMsrHandle handle, CeboMsrIo ioId, CeboMsrCounterConfigId *counterConfigId);
CeboMsrErrorIndex DLLEXPORT CeboMsrSetCounterEnable(CeboMsrHandle handle, CeboMsrIo ioId, int enable);
CeboMsrErrorIndex DLLEXPORT CeboMsrGetCounterEnable(CeboMsrHandle handle, CeboMsrIo ioId, int *enable);

/// Input specific.
CeboMsrErrorIndex DLLEXPORT CeboMsrCalculateMaxBufferedInputFrames(CeboMsrHandle handle, int *frameCount);
CeboMsrErrorIndex DLLEXPORT CeboMsrSetupInputFrame(CeboMsrHandle handle, CeboMsrIo *ios, int count);
CeboMsrErrorIndex DLLEXPORT CeboMsrStartDataAcquisition(CeboMsrHandle handle, int externalStarted, float frameRate, int frameCount, float *correctedFrameRate);
CeboMsrErrorIndex DLLEXPORT CeboMsrStopDataAcquisition(CeboMsrHandle handle);
CeboMsrErrorIndex DLLEXPORT CeboMsrReadBlocking(CeboMsrHandle handle, VoltageArray vbuffer, DigitalArray buffer, int frameCount);
CeboMsrErrorIndex DLLEXPORT CeboMsrReadNonBlocking(CeboMsrHandle handle, VoltageArray vbuffer, DigitalArray buffer, int *transferred);
CeboMsrErrorIndex DLLEXPORT CeboMsrReadDigitalPort(CeboMsrHandle handle, CeboMsrIo ioId, int *value);
CeboMsrErrorIndex DLLEXPORT CeboMsrReadAnalogInput(CeboMsrHandle handle, CeboMsrIo ioId, float *value);
CeboMsrErrorIndex DLLEXPORT CeboMsrReadFrame(CeboMsrHandle handle, VoltageArray vbuffer, DigitalArray buffer);
CeboMsrErrorIndex DLLEXPORT CeboMsrReadCounter(CeboMsrHandle handle, CeboMsrIo ioId, int64_t *counter);

/// Output specific.
CeboMsrErrorIndex DLLEXPORT CeboMsrSetupOutputFrame(CeboMsrHandle handle, CeboMsrIo *ios, int count);
CeboMsrErrorIndex DLLEXPORT CeboMsrWriteDigitalPort(CeboMsrHandle handle, CeboMsrIo ioId, int value);
CeboMsrErrorIndex DLLEXPORT CeboMsrWriteAnalogOutput(CeboMsrHandle handle, CeboMsrIo ioId, float value);
CeboMsrErrorIndex DLLEXPORT CeboMsrWriteFrame(CeboMsrHandle handle, VoltageArray vbuffer, DigitalArray buffer);

#ifdef	__cplusplus
} // extern "C"
#endif

#endif // __CEBOMSR_INTERFACE_H__
