package com.cesys.cebo.cebomsr;

/**
 * Values to be used in enumeration.
 *
 * @author Thomas Hoppe
 */
public final class DeviceType {
	private final int _interfaceId;
	private final String _name;

	int getInterfaceId() {
		return _interfaceId;
	}

	/**
	 * Return instance name, "Usb", "All", ...
	 */
	public String getName() {
		return _name;
	}

	private DeviceType(int interfaceId, String name) {
		_interfaceId = interfaceId;
		_name = name;
	}

	public final static DeviceType All = new DeviceType(SystemConnector.CeboMsrTypeAll, "All");
	public final static DeviceType Usb = new DeviceType(SystemConnector.CeboMsrTypeUsb, "Usb");
	public final static DeviceType CeboLC = new DeviceType(SystemConnector.CeboMsrTypeCeboLC, "CeboLC");
	public final static DeviceType CeboStick = new DeviceType(SystemConnector.CeboMsrTypeCeboStick, "CeboStick");

	static DeviceType getDeviceTypeById(int id) {
		switch (id) {
		case SystemConnector.CeboMsrTypeAll: return All;
		case SystemConnector.CeboMsrTypeUsb: return Usb;
		case SystemConnector.CeboMsrTypeCeboLC: return CeboLC;
		case SystemConnector.CeboMsrTypeCeboStick: return CeboStick;
		}
		throw new IndexOutOfBoundsException();
	}

	public String toString() {
		return getName();
	}
}
